import Foundation

struct OverviewSectionGenerator {

    private let multipleSectionsTemplate: OverviewSectionTemplate

    private let singleSectionsTemplate: OverviewSectionCleanTemplate

    private let generator: ThumbnailListGenerator

    init(factory: TemplateFactory) {
        self.multipleSectionsTemplate = factory.overviewSection
        self.singleSectionsTemplate = factory.overviewSectionClean
        self.generator = ThumbnailListGenerator(factory: factory)
    }

    func generate(sections: [Element], in parent: Element, language: String, sectionItemCount: Int) -> String {
        sections.map { section in
            let metadata = section.localized(for: language)
            let fullUrl = section.fullPageUrl(for: language)
            let relativeUrl = parent.relativePathToFileWithPath(fullUrl)

            var content = [OverviewSectionTemplate.Key : String]()
            content[.url] = relativeUrl
            content[.title] = metadata.title
            content[.items] = generator.generateContent(
                items: section.itemsForOverview(sectionItemCount),
                parent: parent,
                language: language,
                style: section.thumbnailStyle)
            content[.more] = metadata.moreLinkText

            return multipleSectionsTemplate.generate(content)
        }
        .joined(separator: "\n")
    }

    func generate(section: Element, language: String) -> String {
        var content = [OverviewSectionCleanTemplate.Key : String]()
        content[.items] = generator.generateContent(
            items: section.itemsForOverview(),
            parent: section,
            language: language,
            style: section.thumbnailStyle)
        return singleSectionsTemplate.generate(content)
    }
}
